@@ -17,12 +17,12 @@
  */
 package org.apache.tools.ant.taskdefs.launcher;
 
-import java.io.File;
-import java.io.IOException;
-
 import static org.apache.tools.ant.MagicNames.ANT_SHELL_LAUNCHER_REF_ID;
 import static org.apache.tools.ant.MagicNames.ANT_VM_LAUNCHER_REF_ID;
 
+import java.io.File;
+import java.io.IOException;
+
 import org.apache.tools.ant.Project;
 import org.apache.tools.ant.taskdefs.condition.Os;
 import org.apache.tools.ant.types.Commandline;
@@ -80,7 +80,7 @@ public class CommandLauncher {
 
     /**
      * Launches the given command in a new process.
-     * 
+     *
      * @param project
      *        The project that the command is part of.
      * @param cmd
@@ -104,7 +104,7 @@ public class CommandLauncher {
     /**
      * Launches the given command in a new process, in the given
      * working directory.
-     * 
+     *
      * @param project
      *        The project that the command is part of.
      * @param cmd
@@ -175,16 +175,13 @@ public class CommandLauncher {
             try {
                 launcher = (CommandLauncher) Class.forName(launcherClass)
                     .newInstance();
-            }
-            catch(InstantiationException e) {
+            } catch(InstantiationException e) {
                 System.err.println("Could not instantiate launcher class "
                                    + launcherClass + ": " + e.getMessage());
-            }
-            catch(IllegalAccessException e) {
+            } catch(IllegalAccessException e) {
                 System.err.println("Could not instantiate launcher class "
                                    + launcherClass + ": " + e.getMessage());
-            }
-            catch(ClassNotFoundException e) {
+            } catch(ClassNotFoundException e) {
                 System.err.println("Could not instantiate launcher class "
                                    + launcherClass + ": " + e.getMessage());
             }
